from time import sleep
from copreus.baseclasses.adriver import ADriver
from copreus.baseclasses.apolling import APolling
from copreus.baseclasses.aspi import ASPI
from copreus.baseclasses.calibratedvalue import CalibratedValue
from copreus.schema.adc import get_schema


class ADC(ADriver, APolling, ASPI):
    """Driver for ADC (analog-digital-converter) that are connected via spi (e.g. TLC548).

    The driver entry in the yaml file consists of:
      * ADriver entries
        * topics_sub: readnow - mqtt-translations.readnow
        * topics_pub: raw (=integer value returned from adc), volt (=converted and calibrated value)
      * APolling entries
      * ASPI entries
      * CalibratedValue entries in a sub-block named 'calibration'
      * ADC own entries are
        * maxvalue: maximum value in volt. result will be normalized towards this value.
        * bit: how many bits are used. typical values are 8, 10, and 12.

    Example:
    driver:
        type: adc
        spi:
            pin_cs: 17
            bus: 0
            device: 1
            maxspeed: 500000
        topics-pub:
            raw: /adc/raw
            volt: /adc/volt
        topics-sub:
            poll-now: /adc/pollnow
        mqtt-translations:
            poll-now: True
        maxvalue: 24
        bit: 8
        poll-interval: 30
        calibration:
            use-calibration: False
            values:
            # - [ref_value, raw_value]
              - [0, 0]
              - [7.2, 6.4]
              - [24, 24]
    """

    _max_value = -1  # maximum value in volt
    _resolution = -1  # 2**bit
    _calibrated_value = None  # copreus.baseclasses.CalibratedValue

    def __init__(self, config, mqtt_client=None, logger=None, spi_lock=None, stdout_log_level=None, no_gui=None,
                 manage_monitoring_agent=True):
        ADriver.__init__(self, config, mqtt_client, logger, logger_name=self.__class__.__name__,
                         stdout_log_level=stdout_log_level, no_gui=no_gui,
                         manage_monitoring_agent=manage_monitoring_agent)
        APolling.__init__(self, self._config, self._mqtt_client, self._logger)
        ASPI.__init__(self, self._config, self._logger, spi_lock)

        self._max_value = float(self._config["maxvalue"])
        self._resolution = 2**int(self._config["bit"])
        self._calibrated_value = CalibratedValue(self._logger, self._config["calibration"],
                                                 self._max_value / float(self._resolution))

        self._ui_commands["poll"] = self._cmd_poll

    def _cmd_poll(self, args):
        """poll - polls the ADC and outputs the current value: POLL"""
        raw, value = self._get_values()
        print("[{}]: {} V; raw: {}".format(self._name, value, raw))

    def _poll_device(self):
        """APolling._poll_device"""
        raw, value = self._get_values()
        self._publish_value(self._topics_pub["raw"], raw)
        self._publish_value(self._topics_pub["volt"], value)

    def _get_values(self):
        """polls the adc and returns the raw and the calibrated value"""
        self._transfer([0xFF])[0]  # the adc TLC548 has a two step pipeline -> the first read provides the
                                  # value from the last read. this could be a long time ago.
                                  # -> read twice each time -> digitalization and transfer are
                                  # within a guaranteed short period.
        sleep(0.01)
        raw = self._transfer([0xFF])[0]
        value = self._calibrated_value.value(raw)

        return raw, value

    def _driver_start(self):
        """ADriver._driver_start"""
        self._connect_spi()
        self._start_polling()

    def _driver_stop(self):
        """ADriver._driver_stop"""
        self._stop_polling()
        self._disconnect_spi()

    @classmethod
    def _get_schema(cls):
        return get_schema()

    def _runtime_information(self):
        return {}

    def _config_information(self):
        return {}


def standalone():
    """Calls the static method ADC.standalone()."""
    ADC.standalone()


if __name__ == "__main__":
    ADC.standalone()



import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import _thread
from time import sleep

from copreus.drivers.dac import DAC
from tests_manual.tools import send_to_topic, get_topics_sub, get_resolution, get_maxvalue


filename = '../tests_unit/config_dac.yaml'


def loop(topics):
    while 1:
        topic = topics["raw"]
        span = get_resolution(filename)
        steps = 10
        stepsize = float(span) / float(steps)
        for i in range(0, steps+1):
            value = int(stepsize * i)
            send_to_topic(filename, topic, value)
            sleep(1)

        topic = topics["volt"]
        span = get_maxvalue(filename)
        steps = 10
        stepsize = float(span) / float(steps)
        for i in range(0, steps+1):
            value = (stepsize * i)
            send_to_topic(filename, topic, value)
            sleep(1)


subs = get_topics_sub(filename)
_loop_thread = _thread.start_new_thread(loop, (subs,))
DAC.standalone(('-c', filename))




#from copreus.baseclasses.apolling import APolling
#from copreus.baseclasses.adriver import ADriver
#from copreus.baseclasses.aevents import AEvents
#from copreus.baseclasses.aspi import ASPI
#from copreus.baseclasses.calibratedvalue import CalibratedValue
#from copreus.baseclasses.mymqttclient import MyMQTTClient



import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from time import sleep

from PIL import Image

from copreus.baseclasses.aepaper import EPaperMQTTMessageConverter
from tests_manual.tools import get_mqtt

print("send the given png to the epaper")

filename = 'config_epaperdirect.yaml'

mqtt = get_mqtt(filename)
mqtt.connect()

png = Image.open("gui_v2.png")
msg_png = EPaperMQTTMessageConverter.to_full_image(png)

print("send png image")
mqtt.client.publish("/w/wohnzimmer/thermostat/display/full_image_twice", msg_png)

sleep(2)

mqtt.disconnect()
print("done")



def get_schema():
    basic_aeschema_properties = {
        "model": {
            "description": "display-type. one of the keys of EPaperMQTTMessageConverter.size. [1.54, 2.13, 2.9, ...]",
            "type": "number",
            "enum": [1.54, 2.13, 2.9]

        },
        "transpose": {
            "description": "0/90/180/270 rotation applied to all received images.",
            "type": "integer",
            "enum": [0, 90, 180, 270]
        },
        "pin_rst": {
            "description": "reset gpio pin. according to data sheet",
            "type": "integer"
        },
        "pin_dc": {
            "description": "data/command pin. flag if value is to be interpreted as command or as data (see "
                           "data sheet).",
            "type": "integer"
        },
        "pin_busy": {
            "description": "busy flag. set by epaper (see data sheet).",
            "type": "integer"
        },
        "autodeepsleep": {
            "description": "if true, epaper is set to deep sleep after each update.",
            "type": "boolean"
        },
        "vcom": {
            "description": "set vcom supply to given level. (-0.2steps - -3.3 in 0.1V steps; value used in waveform "
                           "demo is -3.2V)",
            "type": "number",
            "minimum": -3.3,
            "maximum": -0.2
        },
    }

    return basic_aeschema_properties



import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from copreus.drivers.dht import DHT
from pelops.myconfigtools import read_config
from pelops.mymqttclient import MyMQTTClient
from pelops.mylogger import create_logger


def input_handler_temperature(message):
    print("temperature: {}".format(message))


def input_handler_humidity(message):
    print("humidity: {}".format(message))


filename = '../tests_unit/config_dht.yaml'
config = read_config(filename)
logger = create_logger(config["logger"], __name__)
mqttclient = MyMQTTClient(config["mqtt"], logger, True)
mqttclient.connect()
mqttclient.subscribe(config["driver"]["topics-pub"]["temperature"], input_handler_temperature)
mqttclient.subscribe(config["driver"]["topics-pub"]["humidity"], input_handler_humidity)


DHT.standalone(('-c', filename))



from threading import Lock


class AI2C:
    _i2c_lock = None  # threading.Lock for i2c interface access

    def __init__(self, i2c_lock):
        if i2c_lock is None:
            self._i2c_lock = Lock()
        else:
            self._i2c_lock = i2c_lock



from time import time
import RPi.GPIO as GPIO
from copreus.baseclasses.adriver import ADriver
from copreus.baseclasses.aevents import AEvents
from copreus.schema.rotaryencoder import get_schema
import enum
import collections
import pelops.ui.tools


class Direction(enum.Enum):
    NONE = 0
    LEFT = -1
    RIGHT = 1


EntryType = collections.namedtuple('EntryType', ['time', 'direction'])


class ARotaryEncoder(ADriver, AEvents):
    """Driver for rotary encoder like the KY-040 with software solutions for debouncing and direction detection.

    The driver entry in the yaml file consists of:
      * ADriver entries
        * topics_pub:
          * rotate - mqtt-translations.rotate-left and mqtt-translations.rotate-right
          * button_pressed - mqtt-translations.button_pressed
          * button_state - mqtt-translations.button_state-open and mqtt-translations.button_state-closed
      * RotaryEncoder entries
        * sw: gpio pin for pressing the rotary encoder
        * dt: dt gpio pin
        * clk: clk gpio pin

    Example:
    driver:
        type: rotaryencoder
        pin_sw:  12
        pin_dt:  16
        pin_clk: 20
        topics-pub:
            rotate: /input1/rotate
            button_pressed: /input1/button/pressed
            button_state:   /input1/button/state
        mqtt-translations:
            rotate-left: LEFT
            rotate-right: RIGHT
            button_pressed: PRESSED
            button_state-open: OPEN
            button_state-closed: CLOSED
    """

    _clk = -1  # clk gpio pin id
    _dt = -1  # dt gpio pin id
    _sw = -1  # sw gpio pin id

    _rotation_history = None  # stores the last n detected rotations

    def __init__(self, config, mqtt_client=None, logger=None, logger_name=None, stdout_log_level=None, no_gui=None,
                 manage_monitoring_agent=True):
        ADriver.__init__(self, config, mqtt_client, logger,
                         stdout_log_level=stdout_log_level, no_gui=no_gui,
                         manage_monitoring_agent=manage_monitoring_agent)
        AEvents.__init__(self, self._config, self._logger)

        self._clk = self._config["pin_clk"]
        self._dt = self._config["pin_dt"]
        self._sw = self._config["pin_sw"]

        self._add_event(self._clk, self._callback_rotary, 1)
        self._add_event(self._sw, self._callback_sw)

        self._rotation_history = collections.deque(maxlen=25)

        self._ui_commands["button_state"] = self._cmd_button_state

    def _cmd_rotation_history(self, args):
        """rotation_history - displays the last rotation events (time + direction): ROTATION_HISTORY"""
        text = "[{}] rotation history\n".format(self._name)
        now = time()
        for entry in self._rotation_history:
            t = now - entry.time
            d = entry.direction
            text += " - {} s; {}".format(t, d.name)
        pelops.ui.tools.more(text)

    def _cmd_button_state(self, args):
        """button_state - displays if the button is closed or open: BUTTON_STATE"""
        state = GPIO.input(self._sw)
        if not state:
            print("[{}] button (gpio: {}) is closed".format(self._name, self._sw))
        else:
            print("[{}] button (gpio: {}) is open".format(self._name, self._sw))

    def _callback_sw(self, channel):
        """Event handler for gpio pin 'sw'. Publishes to the topics 'button_state' and 'button_pressed'."""
        self._logger.info("SW-event detected.")
        state = GPIO.input(self._sw)
        if not state:
            self._publish_value(self._topics_pub["button_pressed"], self._mqtt_translations["button_pressed"])
            self._publish_value(self._topics_pub["button_state"], self._mqtt_translations["button_state-closed"])
        else:
            self._publish_value(self._topics_pub["button_state"], self._mqtt_translations["button_state-open"])

    def _callback_rotary(self, channel):
        """rotation detection and publishing"""
        rotate = self._detect_rotation_direction()
        if rotate == Direction.RIGHT:
            self._publish_value(self._topics_pub["rotate"], self._mqtt_translations["rotate-right"])
            self._rotation_history.append(EntryType(time=time(), direction=rotate))
        elif rotate == Direction.LEFT:
            self._publish_value(self._topics_pub["rotate"], self._mqtt_translations["rotate-left"])
            self._rotation_history.append(EntryType(time=time(), direction=rotate))
        else:
            pass

    def _detect_rotation_direction(self):
        raise NotImplementedError

    def _driver_start(self):
        """ADriver._driver_start"""
        GPIO.setmode(GPIO.BCM)
        GPIO.setup(self._dt, GPIO.IN, pull_up_down=GPIO.PUD_UP)
        GPIO.setup(self._sw, GPIO.IN, pull_up_down=GPIO.PUD_UP)
        GPIO.setup(self._clk, GPIO.IN, pull_up_down=GPIO.PUD_UP)
        self._last_clk = GPIO.input(self._clk)
        self._register_events()

    def _driver_stop(self):
        """ADriver._driver_stop"""
        self._unregister_events()
        GPIO.cleanup(self._dt)
        GPIO.cleanup(self._sw)
        GPIO.cleanup(self._clk)

    @classmethod
    def _get_schema(cls):
        return get_schema()

    def _runtime_information(self):
        return {}

    def _config_information(self):
        return {}




from copreus.baseclasses.adriver import ADriver
from copreus.baseclasses.aspi import ASPI
from copreus.baseclasses.calibratedvalue import CalibratedValue
from copreus.schema.dac import get_schema


class DAC(ADriver, ASPI):
    """Driver for ADC (analog-digital-converter) that are connected via spi (e.g. MCP4922).

    The driver entry in the yaml file consists of:
      * ADriver entries
        * topics_sub: raw (=integer value for the dac), volt (=converted and calibrated value)
      * ASPI entries
      * CalibratedValue entries in a sub-block named 'calibration'
      * ADC own entries are
        * maxvalue: maximum value in volt. volt will be normalized towards this value.
        * bit: how many bits are used. typical values are 8, 10, and 12.
        * config_dac: configuration bit-sequence according to datasheet (0 if none)

    Example:
    driver:
        type: dac
        config_dac: 0x3000
        spi:
            pin_cs: 4
            bus: 0
            device: 1
            maxspeed: 500000
        topics-sub:
            raw: /dac/raw
            volt: /dac/volt
        maxvalue: 24
        bit: 12
        calibration:
            use-calibration: True
            values:
            # - [ref_value, raw_value]
              - [0.0, 0.0]
              - [7.23, 6.0]
              - [24, 24]
    """

    _max_value = -1  # maximum value in volt
    _config_dac = -1  # configuration bit sequence according to data sheet
    _max_raw = -1  # 2**bit - 1
    _calibrated_value = None  # copreus.baseclasses.CalibratedValue

    def __init__(self, config, mqtt_client=None, logger=None, spi_lock=None, stdout_log_level=None, no_gui=None,
                 manage_monitoring_agent=True):
        ADriver.__init__(self, config, mqtt_client, logger, logger_name=self.__class__.__name__,
                         stdout_log_level=stdout_log_level, no_gui=no_gui,
                         manage_monitoring_agent=manage_monitoring_agent)
        ASPI.__init__(self, self._config, self._logger, spi_lock)

        self._config_dac = self._config["config_dac"]

        self._max_value = self._config["maxvalue"]
        self._max_raw = (2 ** int(self._config["bit"]))-1
        self._calibrated_value = CalibratedValue(self._logger, self._config["calibration"],
                                                 self._max_value / float(self._max_raw+1))

        self._mqtt_client.subscribe(self._topics_sub["raw"], self._write_raw)
        self._mqtt_client.subscribe(self._topics_sub["volt"], self._write_volt)

        self._ui_commands["write_raw"] = self._cmd_write_raw
        self._ui_commands["write_volt"] = self._cmd_write_volt

    def _cmd_write_volt(self, args):
        """write_volt float_value: write the provided float value (0-{}) to the DAC: WRITE_VOLT 123.4"""\
            .format(self._max_value)

        if len(args) != 1:
            print("wrong number of arguments. expected 'write_raw int_value'")
        else:
            try:
                value = float(args[0])
                if not (0 <= value <= self._max_value):
                    print("value {} not in range 0<=x<={}.".format(value, self._max_value))
                else:
                    raw = self._calibrated_value.raw(value)
                    self._transfer_raw(raw)
                    print("[{}] written volt value {} to dac (raw: {})".format(self._name, value, rae))
            except ValueError:
                print("argument must be a valid float. got '{}'".format(args[0]))

    def _cmd_write_raw(self, args):
        """write_raw int_value: write the provided integer value (0-{}) to the DAC: WRITE_RAW 123"""\
            .format(self._max_raw)

        if len(args) != 1:
            print("wrong number of arguments. expected 'write_raw int_value'")
        elif not args[0].isdigits():
            print("argument must be a valid integer. got '{}'".format(args[0]))
        elif not (0 <= int(args[0]) <= self._max_raw):
            print("value {} not in range 0<=x<={}.".format(int(args[0]), self._max_raw))
        else:
            value = int(args[0])
            self._transfer_raw(value)
            print("[{}] written raw value {} to dac".format(self._name, value))

    def _write_raw(self, msg):
        """on_message handler for topic sub 'raw'."""
        self._logger.info("received message '{}' on topic '{}'.".format(msg, self._topics_sub["raw"]))
        self._transfer_raw(int(msg))

    def _write_volt(self, msg):
        """on_message handler for topic sub 'volt'."""
        self._logger.info("received message '{}' on topic '{}'.".format(msg, self._topics_sub["volt"]))
        volt = float(msg)
        raw = self._calibrated_value.raw(volt)
        self._transfer_raw(raw)

    def _transfer_raw(self, raw):
        """Transfer the raw integer value (0<=value<2**bit) to the dac."""
        output = self._config_dac
        if raw > self._max_raw:
            raw = self._max_raw
        if raw < 0:
            raw = 0
        output |= raw
        buf0 = (output >> 8) & 0xff
        buf1 = output & 0xff
        self._logger.info("sending [{},{}] to dac.".format(buf0, buf1))
        self._transfer([buf0, buf1])

    def _driver_start(self):
        """ADriver._driver_start"""
        self._connect_spi()

    def _driver_stop(self):
        """ADriver._driver_stop"""
        self._disconnect_spi()

    @classmethod
    def _get_schema(cls):
        return get_schema()

    def _runtime_information(self):
        return {}

    def _config_information(self):
        return {}


def standalone():
    """Calls the static method DAC.standalone()."""
    DAC.standalone()


if __name__ == "__main__":
    DAC.standalone()



import RPi.GPIO as GPIO
from copreus.baseclasses.adriver import ADriver
from copreus.baseclasses.aevents import AEvents
from copreus.schema.input import get_schema


class Input(ADriver, AEvents):
    """Generic driver that waits for events on the given input pin.

    The driver entry in the yaml file consists of:
      * ADriver entries
        * topics_pub:
          * button_pressed - mqtt-translations.button_pressed
          * button_state - mqtt-translations.button_state-open and mqtt-translations.button_state-closed
      * Input entries
        * pin: gpio pin

    Example:
    driver:
        type: input
        pin:  21
        topics-pub:
            button_pressed: /test/button/pressed
            button_state:   /test/button/state
        mqtt-translations:
            button_pressed: PRESSED
            button_state-open: OPEN
            button_state-closed: CLOSED

    """

    _pin = -1  # gpio pin id

    def __init__(self, config, mqtt_client=None, logger=None, stdout_log_level=None, no_gui=None,
                 manage_monitoring_agent=True):
        ADriver.__init__(self, config, mqtt_client, logger, logger_name=self.__class__.__name__,
                         stdout_log_level=stdout_log_level, no_gui=no_gui,
                         manage_monitoring_agent=manage_monitoring_agent)
        AEvents.__init__(self, self._config, self._logger)

        self._pin = self._config["pin"]
        self._add_event(self._pin, self._callback_pin)
        self._ui_commands["gpio_state"] = self._cmd_gpio_state

    def _cmd_gpio_state(self, args):
        """gpio_state - displays if the gpio is closed or open: GPIO_STATE"""
        state = GPIO.input(self._pin)
        if not state:
            print("[{}] gpio {} is closed".format(self._name, self._pin))
        else:
            print("[{}] gpio {} is open".format(self._name, self._pin))

    def _callback_pin(self, channel):
        """Event handler for gpio pin 'pin'. Publishes to the topics 'button_state' and 'button_pressed'."""
        state = GPIO.input(self._pin)
        self._logger.info("Input._callback_pin - received event. pin state: {}.".format(state))
        if not state:
            self._publish_value(self._topics_pub["button_pressed"], self._mqtt_translations["button_pressed"])
            self._publish_value(self._topics_pub["button_state"], self._mqtt_translations["button_state-closed"])
        else:
            self._publish_value(self._topics_pub["button_state"], self._mqtt_translations["button_state-open"])

    def _driver_start(self):
        """ADriver._driver_start"""
        GPIO.setmode(GPIO.BCM)
        GPIO.setup(self._pin, GPIO.IN, pull_up_down=GPIO.PUD_UP)
        self._register_events()

    def _driver_stop(self):
        """ADriver._driver_stop"""
        self._unregister_events()
        GPIO.cleanup(self._pin)

    @classmethod
    def _get_schema(cls):
        return get_schema()

    def _runtime_information(self):
        return {}

    def _config_information(self):
        return {}


def standalone():
    """Calls the static method Input.standalone()."""
    Input.standalone()


if __name__ == "__main__":
    Input.standalone()
